@@ -260,6 +260,55 @@ class SubscriptionManagerIsSubscribedGroupTest(TestCase):
         self.assertFalse(is_subscribed)
 
 
+class SubscriptionFilterNotSubscribedTest(TestCase):
+    def setUp(self):
+        self.super_ct = G(ContentType)
+        self.sub_ct = G(ContentType)
+        self.super_e1 = G(Entity, entity_type=self.super_ct)
+        self.super_e2 = G(Entity, entity_type=self.super_ct)
+        self.sub_e1 = G(Entity, entity_type=self.sub_ct)
+        self.sub_e2 = G(Entity, entity_type=self.sub_ct)
+        self.sub_e3 = G(Entity, entity_type=self.sub_ct)
+        self.sub_e4 = G(Entity, entity_type=self.sub_ct)
+        self.ind_e1 = G(Entity, entity_type=self.sub_ct)
+        self.ind_e2 = G(Entity, entity_type=self.sub_ct)
+        self.medium = G(Medium)
+        self.source = G(Source)
+        G(EntityRelationship, sub_entity=self.sub_e1, super_entity=self.super_e1)
+        G(EntityRelationship, sub_entity=self.sub_e2, super_entity=self.super_e1)
+        G(EntityRelationship, sub_entity=self.sub_e3, super_entity=self.super_e2)
+        G(EntityRelationship, sub_entity=self.sub_e4, super_entity=self.super_e2)
+
+    def test_group_and_individual_subscription(self):
+        G(Subscription, entity=self.ind_e1, source=self.source, medium=self.medium, subentity_type=None)
+        G(Subscription, entity=self.super_e1, source=self.source, medium=self.medium, subentity_type=self.sub_ct)
+        entities = [self.sub_e1, self.sub_e3, self.ind_e1, self.ind_e2]
+        filtered_entities = Subscription.objects.filter_not_subscribed(self.source, self.medium, entities)
+        expected_entity_ids = [self.sub_e1.id, self.ind_e1.id]
+        self.assertEqual(set(filtered_entities.values_list('id', flat=True)), set(expected_entity_ids))
+
+    def test_unsubscribe_filtered_out(self):
+        G(Subscription, entity=self.ind_e1, source=self.source, medium=self.medium, subentity_type=None)
+        G(Subscription, entity=self.super_e1, source=self.source, medium=self.medium, subentity_type=self.sub_ct)
+        G(Unsubscribe, entity=self.sub_e1, source=self.source, medium=self.medium)
+        entities = [self.sub_e1, self.sub_e2, self.sub_e3, self.ind_e1, self.ind_e2]
+        filtered_entities = Subscription.objects.filter_not_subscribed(self.source, self.medium, entities)
+        expected_entity_ids = [self.sub_e2.id, self.ind_e1.id]
+        self.assertEqual(set(filtered_entities.values_list('id', flat=True)), set(expected_entity_ids))
+
+    def test_entities_not_passed_in_filtered(self):
+        G(Subscription, entity=self.ind_e1, source=self.source, medium=self.medium, subentity_type=None)
+        G(Subscription, entity=self.super_e1, source=self.source, medium=self.medium, subentity_type=self.sub_ct)
+        entities = list(self.super_e1.get_sub_entities().is_any_type(self.sub_ct))
+        filtered_entities = Subscription.objects.filter_not_subscribed(self.source, self.medium, entities)
+        self.assertEqual(set(filtered_entities), set(entities))
+
+    def test_different_entity_types_raises_error(self):
+        entities = [self.sub_e1, self.super_e1]
+        with self.assertRaises(ValueError):
+            Subscription.objects.filter_not_subscribed(self.source, self.medium, entities)
+
+
 class UnsubscribeManagerIsUnsubscribed(TestCase):
     def test_is_unsubscribed(self):
         entity, source, medium = G(Entity), G(Source), G(Medium)
@@ -322,3 +371,7 @@ class NumberOfQueriesTests(TestCase):
 
         with self.assertNumQueries(1):
             Subscription.objects._is_subscribed_group(source=s1, medium=m1, entity=e6, subentity_type=ct)
+
+        with self.assertNumQueries(1):
+            entities = [e0, e1]
+            list(Subscription.objects.filter_not_subscribed(source=s1, medium=m1, entities=entities))
